from ast_grep_py import SgRoot

source = """
function test() {
  let a = 123
}
""".strip()

sg = SgRoot(source, "javascript")
root = sg.root()
node1 = root.find(pattern="let $A = $B")
assert node1 is not None
node2 = root.find(pattern="let $A = 123")
assert node2 is not None

def test_one_fix():
    edit = node1.replace("let b = 456")
    r = node1.range()
    assert edit.start_pos == r.start.index
    assert edit.end_pos == r.end.index
    s = node1.commit_edits([edit])
    assert s == "let b = 456"
    s = root.commit_edits([edit])
    assert s == "function test() {\n  let b = 456\n}"

def test_multiple_fix():
    sg = SgRoot('いいよ = log(123) + log(456)', "javascript")
    root = sg.root()
    nodes = root.find_all(kind="number")
    edits = [node.replace('114514') for node in nodes]
    edits = sorted(edits, key=lambda x: x.start_pos, reverse=True)
    s = root.commit_edits(edits)
    assert s == "いいよ = log(114514) + log(114514)"

def test_change_unicode():
    sg = SgRoot('いいよ = log(こいよ)', "javascript")
    root = sg.root()
    nodes = root.find_all(kind="identifier")
    edits = [node.replace('114514') for node in nodes]
    s = root.commit_edits(edits)
    assert s == "114514 = 114514(114514)"

def test_modified_fix():
    sg = SgRoot('いいよ = log(514)', "javascript")
    root = sg.root()
    node = root.find(kind="number")
    assert node
    edit = node.replace('こいよ')
    edit.start_pos -= 1
    edit.end_pos += 1
    s = root.commit_edits([edit])
    assert s == "いいよ = logこいよ"


from typing import List, Optional, overload, Unpack

from . import Rule, Config

class Pos:
    line: int
    column: int
    index: int

class Range:
    start: Pos
    end: Pos

class Edit:
    start_pos: int
    end_pos: int
    inserted_text: str

class SgRoot:
    def __init__(self, src: str, language: str) -> None: ...
    def root(self) -> SgNode: ...
    def filename(self) -> str: ...

class SgNode:
    # Node Inspection
    def range(self) -> Range: ...
    def is_leaf(self) -> bool: ...
    def is_named(self) -> bool: ...
    def is_named_leaf(self) -> bool: ...
    def kind(self) -> str: ...
    def text(self) -> str: ...

    # Refinement
    def matches(self, **rule: Unpack[Rule]) -> bool: ...
    def inside(self, **rule: Unpack[Rule]) -> bool: ...
    def has(self, **rule: Unpack[Rule]) -> bool: ...
    def precedes(self, **rule: Unpack[Rule]) -> bool: ...
    def follows(self, **rule: Unpack[Rule]) -> bool: ...
    def get_match(self, meta_var: str) -> Optional[SgNode]: ...
    def get_multiple_matches(self, meta_var: str) -> List[SgNode]: ...
    def get_transformed(self, meta_var: str) -> Optional[str]: ...
    def __getitem__(self, meta_var: str) -> SgNode: ...

    # Search
    @overload
    def find(self, config: Config) -> Optional[SgNode]: ...
    @overload
    def find(self, **kwargs: Unpack[Rule]) -> Optional[SgNode]: ...
    @overload
    def find_all(self, config: Config) -> List[SgNode]: ...
    @overload
    def find_all(self, **kwargs: Unpack[Rule]) -> List[SgNode]: ...

    # Tree Traversal
    def get_root(self) -> SgRoot: ...
    def field(self, name: str) -> Optional[SgNode]: ...
    def parent(self) -> Optional[SgNode]: ...
    def child(self, nth: int) -> Optional[SgNode]: ...
    def children(self) -> List[SgNode]: ...
    def ancestors(self) -> List[SgNode]: ...
    def next(self) -> Optional[SgNode]: ...
    def next_all(self) -> List[SgNode]: ...
    def prev(self) -> Optional[SgNode]: ...
    def prev_all(self) -> List[SgNode]: ...

    # Edit
    def replace(self, new_text: str) -> Edit: ...
    def commit_edits(self, edits: List[Edit]) -> str: ...